@@ -37,8 +37,6 @@ extern "C" {
 }
 #endif
 
-#ifndef SIMPLE_CODE
-
 #include <netinet/in.h>
 
 #include <fstream>
@@ -83,13 +81,8 @@ extern "C" {
 
 #include "Compass.h"
 #include "TimeLine.h"
-#endif
-#include <iostream>
 
-#ifdef SIMPLE_CODE
-GMainContext *context;
-GMainLoop    *loop = NULL;
-#else
+#include <iostream>
 
 static long obsoleteDataAge = 30; // maximum ge of data in seconds until value is invalid
 static long reconnectTime   = 10; // Period of time after which we check for disconnection and
@@ -479,17 +472,21 @@ static MyWindow*         window=NULL;
 static Lum::Def::AppInfo info;
 static unsigned long     trackingFileId='G'*256*256*256+'J'*256*256+'T'*256+'\0';
 static unsigned long     trackingFileVersion=1;
-#endif
 
 #if defined(HAVE_LIB_GPS)
 static void cb( struct gps_data_t * data,
                char * buf,
                size_t len,
                int level );
+#elif defined(HAVE_LIB_LOCATION)
+static void gps_location_changed(LocationGPSDevice *device, gpointer userdata);
+static void gps_location_connected(LocationGPSDevice *new_device, gpointer userdata);
+static void gps_location_disconnected(LocationGPSDevice *new_device, gpointer userdata);
+static void gps_location_started(LocationGPSDControl *control, gpointer userdata);
+static void gps_location_stopped(LocationGPSDControl *control, gpointer userdata);
+static void gps_location_error(LocationGPSDControl *control, gpointer userdata);
 #endif
 
-#ifndef SIMPLE_CODE
-
 std::wstring DoubleToString(double value)
 {
   if (!isnan(value)) {
@@ -605,11 +602,20 @@ private:
   std::ofstream            trackFile;
 
 #if defined(HAVE_LIB_LOCATION)
+  LocationGPSDevice        *device;
   LocationGPSDControl      *control;
 #endif
 #if defined(HAVE_LIB_GPS)
   struct gps_data_t        *gpsData;
   pthread_t                gpsThread;
+#elif defined(HAVE_LIB_LOCATION)
+    guint                  idd_changed;
+    guint                  idd_con;
+    guint                  idd_disc;
+	
+    guint                  idc_run;
+    guint                  idc_stop;
+    guint                  idc_error;
 #endif
 
 public:
@@ -629,13 +635,49 @@ private:
 
   void InitializeGPS()
   {
-    std::cout << "Initialize GPS..." << std::endl;
+      std::cout << "Initializing GPS..." << std::endl;
 #if defined(HAVE_LIB_LOCATION)
+    std::cout << "Initialize LocationGPSDcontrol..." << std::endl;
     control=location_gpsd_control_get_default();
 
-    location_gpsd_control_start(control);
+    if (control!=NULL) {
+#if defined(LOCATION_METHOD_USER_SELECTED)
+      std::cout << "Initializing LocationGPSDevice..." << std::endl;
+      device=(LocationGPSDevice*)g_object_new(LOCATION_TYPE_GPS_DEVICE, NULL);
+
+      if (device!=NULL) {
+        /* Install signal handlers */
+        idd_changed=g_signal_connect(device,"changed",G_CALLBACK(gps_location_changed),NULL);
+        idd_con=g_signal_connect(device,"connected",G_CALLBACK(gps_location_connected),NULL);
+        idd_disc=g_signal_connect(device,"disconnected",G_CALLBACK(gps_location_disconnected),NULL);
+
+        idc_run=g_signal_connect(control,"gpsd_running",G_CALLBACK(gps_location_started),NULL);
+        idc_stop=g_signal_connect(control,"gpsd_stopped",G_CALLBACK(gps_location_stopped),NULL);
+        idc_error=g_signal_connect(control,"error",G_CALLBACK(gps_location_error),NULL);
+      }
+      else {
+        std::cerr << "GPS NOT initialized!" << std::endl;
+      }
+
+      g_object_set(G_OBJECT(control),
+                   "preferred-method",
+                   LOCATION_METHOD_USER_SELECTED,
+                   NULL);
+      g_object_set(G_OBJECT(control),
+                   "preferred-interval",
+                   LOCATION_INTERVAL_5S,
+                   NULL);
 #endif
+      std::cout << "Starting GPS daemon..." << std::endl;
+      location_gpsd_control_start(control);
+    }
+    else {
+      std::cerr << "Cannot start GPS daemon!" << std::endl;
+    }
+#endif
+
 #if defined(HAVE_LIB_GPS)
+    std::cout << "Initializing GPS daemon..." << std::endl;
     // This fixed a bug in libgps, using locale aware functions to parse non-locale-aware float values
     setlocale(LC_NUMERIC,"C");
     gpsData=::gps_open("127.0.0.1","2947");
@@ -646,8 +688,10 @@ private:
       std::cout << "GPS initialized!" << std::endl;
     }
     else {
-      std::cout << "GPS NOT initialized: " << errno << std::endl;
+      std::cerr << "GPS NOT initialized: " << errno << std::endl;
     }
+#elif !defined(HAVE_LIB_LOCATION)
+    std::cerr << "No GPS initialization possible!" << std::endl;
 #endif
   }
 
@@ -660,14 +704,31 @@ private:
       ::gps_close(gpsData);
       gpsData=NULL;
     }
+#elif defined(HAVE_LIB_LOCATION)
+    if (device!=NULL) {
+      g_signal_handler_disconnect(device,idd_changed);
+      g_signal_handler_disconnect(device,idd_con);
+      g_signal_handler_disconnect(device,idd_disc);
+  	
+      g_signal_handler_disconnect(control,idc_run);
+      g_signal_handler_disconnect(control,idc_stop);
+      g_signal_handler_disconnect(control,idc_error);
+
+      std::cout << "Freeing LocationGPSDevice..." << std::endl;
+      g_object_unref(device);	
+    }
 #endif
 
 #if defined(HAVE_LIB_LOCATION)
-    location_gpsd_control_stop(control);
+    if (control!=NULL) {
+      std::cout << "Freeing LocationGPSDcontrol..." << std::endl;
 
-    /*Device control cleanup */
-    g_object_unref(control);
-    control=NULL;
+      location_gpsd_control_stop(control);
+
+      /*Device control cleanup */
+      g_object_unref(control);
+      control=NULL;
+    }
 #endif
     std::cout << "GPS deinitialized!" << std::endl;
   }
@@ -727,13 +788,22 @@ public:
     reconnectCheckTimer(new Lum::Model::Action()),
     tracking(new Lum::Model::Boolean(false))
 #if defined(HAVE_LIB_LOCATION)
+    ,device(NULL)
     ,control(NULL)
 #endif
 #if defined(HAVE_LIB_GPS)
     ,gpsData(NULL)
+#elif defined(HAVE_LIB_LOCATION)
+    ,idd_changed(0)
+    ,idd_con(0)
+    ,idd_disc(0)
+    ,idc_run(0)
+    ,idc_stop(0)
+    ,idc_error(0)
 #endif
   {
     Observe(GetOpenedAction());
+    Observe(GetClosedAction());
     Observe(startTrackingAction);
     Observe(stopTrackingAction);
     Observe(deleteTrackAction);
@@ -980,9 +1050,9 @@ public:
       tim=data.GetTime();
     }
 
-    unsigned long flon=htonl(round((lon+90.0)*10000000.0));
-    unsigned long flat=htonl(round((lat+180.0)*10000000.0));
-    unsigned long falt=htonl(round(alt*100));
+    unsigned long flon=htonl((unsigned long)rint((lon+90.0)*10000000.0));
+    unsigned long flat=htonl((unsigned long)rint((lat+180.0)*10000000.0));
+    unsigned long falt=htonl((unsigned long)rint(alt*100));
     unsigned long ftim=htonl(tim.GetTime());
 
     //std::cout << "TrackRecord: " << std::hex << flon << " " << flat << " " << falt << " " << ftim << std::endl;
@@ -1490,115 +1560,6 @@ public:
     Lum::OS::display->QueueActionForAsyncNotification(dataChangedAction);
   }
 };
-#endif
-
-#ifdef SIMPLE_CODE
-static void gps_location_changed (LocationGPSDevice *device, gpointer userdata);
-static void gps_location_connected (LocationGPSDevice *new_device, gpointer userdata);
-static void gps_location_disconnected (LocationGPSDevice *new_device, gpointer userdata);
-static void gps_location_started (LocationGPSDControl *control, gpointer userdata);
-static void gps_location_stopped (LocationGPSDControl *control, gpointer userdata);
-static void gps_location_error (LocationGPSDControl *control, gpointer userdata);
-
-static void gps_location_changed(LocationGPSDevice *device, gpointer userdata)
-{
-  std::cout << "gps_location_changed" << std::endl;
-
-    std::cout << "-----------------" << std::endl;
-    std::cout << "Status: " << (device->online? "true" : "false") << std::endl;
-    std::cout << "Detailed status: ";
-    switch (device->status) {
-    case LOCATION_GPS_DEVICE_STATUS_NO_FIX:
-      std::cout << "No fix";
-      break;
-      case LOCATION_GPS_DEVICE_STATUS_FIX:
-      std::cout << "Fix";
-      break;
-    case LOCATION_GPS_DEVICE_STATUS_DGPS_FIX:
-      std::cout << "DGPS fix";
-      break;
-    }
-
-    std::cout << std::endl;
-
-    std::cout << device->satellites_in_view << " satellites in view, " << device->satellites_in_use << " used!" << std::endl;
-
-    /* Fix information */
-    LocationGPSDeviceFix *fix;
-    fix = device->fix;
-
-    std::cout << "Fix mode: ";
-    switch(fix->mode) {
-      case LOCATION_GPS_DEVICE_MODE_NOT_SEEN:
-      std::cout << "No satelite";
-      break;
-    case LOCATION_GPS_DEVICE_MODE_NO_FIX:
-      std::cout << "No fix";
-      break;
-    case LOCATION_GPS_DEVICE_MODE_2D:
-      std::cout << "2D information";
-      break;
-    case LOCATION_GPS_DEVICE_MODE_3D:
-      std::cout << "3D information";
-      break;
-    }
-    std::cout << std::endl;
-
-    if (fix->fields & LOCATION_GPS_DEVICE_TIME_SET) {
-      std::cout << "Time: " << fix->time << " uncertainty: " << fix->ept << std::endl;
-    }
-    if (fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET) {
-      std::cout << "Latitude: " << fix->latitude << " longitude: " << fix->longitude << " uncertainty: " <<  fix->eph << std::endl;
-    }
-    if (fix->fields & LOCATION_GPS_DEVICE_ALTITUDE_SET) {
-      std::cout << "Altitude: " << fix->altitude << " uncertainty: " << fix->epv << std::endl;
-    }
-    if (fix->fields & LOCATION_GPS_DEVICE_SPEED_SET) {
-      std::cout << "Speed: " << fix->speed << " uncertainty: " << fix->eps << std::endl;
-    }
-    if (fix->fields & LOCATION_GPS_DEVICE_TRACK_SET) {
-      std::cout << "Track: " << fix->track << " uncertainty: " << fix->epd << std::endl;
-    }
-    if (fix->fields & LOCATION_GPS_DEVICE_CLIMB_SET) {
-      std::cout << "Climb: " << fix->climb << " uncertainty: " << fix->epc << std::endl;
-    }
-
-    fix = NULL;
-
-    if (device->satellites!=NULL) {
-      for (int i =0; i<device->satellites_in_view; i++) {
-        LocationGPSDeviceSatellite* satellite=(LocationGPSDeviceSatellite*)g_ptr_array_index(device->satellites,i);
-
-        std::cout << "Satellite id:" << satellite->prn << " S/N: " << satellite->signal_strength << " elevation: " << satellite->elevation << " azimuth: " << satellite->azimuth << " " << (satellite->in_use ? "used" : "unused") << std::endl;
-      }
-    }
-}
-
-static void gps_location_connected(LocationGPSDevice *new_device, gpointer userdata) {
-  std::cout << "Connected..." << std::endl;
-}
-
-static void gps_location_disconnected(LocationGPSDevice *new_device, gpointer userdata){
-  std::cout << "disconnected..." << std::endl;
-}
-
-static void gps_location_started(LocationGPSDControl *control, gpointer userdata) {
-  std::cout << "GPSD started!" << std::endl;
-}
-
-static void gps_location_stopped(LocationGPSDControl *control, gpointer userdata) {
-  std::cout << "GPSD stoped!" << std::endl;
-}
-
-static void gps_location_error(LocationGPSDControl *control, gpointer userdata) {
-  std::cout << "GPSD error!" << std::endl;
-}
-
-static void gps_location_quit(int signal) {
-  printf("[Info] SIGINT caught, exiting\n");
-  g_main_loop_quit(loop);
-}
-#endif
 
 #if defined(HAVE_LIB_GPS)
 static void cb(struct gps_data_t* data, char * buf, size_t len, int level)
@@ -1666,9 +1627,86 @@ static void cb(struct gps_data_t* data, char * buf, size_t len, int level)
 
   window->HandleChange();
 }
+#elif defined(HAVE_LIB_LOCATION)
+static void gps_location_changed(LocationGPSDevice *device, gpointer userdata)
+{
+  std::cout << "gps_location_changed!" << std::endl;
+  assert(window!=NULL);
+
+  Lum::OS::WriteGuard<Lum::OS::RWMutex> guard(window->dataMutex);
+
+  GPSData                               &gpsData=window->data;
+
+  gpsData.SetOnline(device->online);
+  gpsData.SetFix(device->status!=LOCATION_GPS_DEVICE_STATUS_NO_FIX);
+
+  /* Fix information */
+  LocationGPSDeviceFix *fix=device->fix;
+
+  if (fix->fields & LOCATION_GPS_DEVICE_TIME_SET) {
+    Lum::Base::SystemTime time(lround(fix->time));
+    gpsData.SetTime(time);
+  }
+  if (fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET && !isnan(fix->latitude)) {
+    gpsData.SetLatitude(fix->latitude);
+  }
+  if (fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET && !isnan(fix->longitude)) {
+    gpsData.SetLatitude(fix->longitude);
+  }
+  if (fix->fields & LOCATION_GPS_DEVICE_ALTITUDE_SET && !isnan(fix->altitude)) {
+    gpsData.SetLatitude(fix->altitude);
+  }
+  if (fix->fields & LOCATION_GPS_DEVICE_SPEED_SET && !isnan(fix->speed)) {
+    gpsData.SetSpeed(fix->speed);
+  }
+  if (fix->fields & LOCATION_GPS_DEVICE_TRACK_SET && !isnan(fix->track)) {
+    gpsData.SetTrack(fix->track);
+  }
+  if (fix->fields & LOCATION_GPS_DEVICE_CLIMB_SET && !isnan(fix->climb)) {
+    gpsData.SetClimb(fix->climb);
+  }
+
+  std::vector<GPSData::Satellite> satellites;
+
+  if (device->satellites!=NULL) {
+    for (size_t i=0; i<device->satellites->len; i++) {
+      LocationGPSDeviceSatellite *sat=(LocationGPSDeviceSatellite*)g_ptr_array_index(device->satellites,i);
+      GPSData::Satellite         s;
+
+      s.id=sat->prn;
+      s.used=sat->in_use;
+      s.snr=sat->signal_strength;
+
+      satellites.push_back(s);
+    }
+  }
+
+  gpsData.SetSatellites(satellites);
+
+  window->HandleChange();
+}
+
+static void gps_location_connected(LocationGPSDevice *new_device, gpointer userdata) {
+  std::cout << "Connected..." << std::endl;
+}
+
+static void gps_location_disconnected(LocationGPSDevice *new_device, gpointer userdata){
+  std::cout << "disconnected..." << std::endl;
+}
+
+static void gps_location_started(LocationGPSDControl *control, gpointer userdata) {
+  std::cout << "GPSD started!" << std::endl;
+}
+
+static void gps_location_stopped(LocationGPSDControl *control, gpointer userdata) {
+  std::cout << "GPSD stoped!" << std::endl;
+}
+
+static void gps_location_error(LocationGPSDControl *control, gpointer userdata) {
+  std::cout << "GPSD error!" << std::endl;
+}
 #endif
 
-#ifndef SIMPLE_CODE
 class Main : public Lum::OS::MainDialog<MyWindow>
 {
 public:
@@ -1686,65 +1724,3 @@ public:
 
 LUM_MAIN(Main,Lum::Base::StringToWString(PACKAGE_NAME))
 
-#else
-
-int main(int argc, char *argv[]) {
-
-	/* Inicialitzacio de la llibreria glib */
-	g_type_init();
-	g_thread_init(NULL);
-	printf("[Info] Glib started\n");
-	
-	
-	/* Location Control setup*/
-	LocationGPSDControl *control = NULL;
-	control = location_gpsd_control_get_default();
-	if(control->can_control)
-		location_gpsd_control_start (control);
-	
-	/* Recupera informacio actual */
-	LocationGPSDevice *device = NULL;
-	device = (LocationGPSDevice*)g_object_new(LOCATION_TYPE_GPS_DEVICE, NULL);
-	location_gps_device_reset_last_known (device);
-	printf("[Debug] Device pointer adress: %x\n", (unsigned int)device);
-	printf("[Info] Device object obtained\n");
-	
-	/* Install signal handlers */
-	signal(SIGINT, gps_location_quit);
-	guint idd_changed = g_signal_connect (device, "changed", G_CALLBACK (gps_location_changed), NULL);
-	guint idd_con = g_signal_connect (device, "connected", G_CALLBACK (gps_location_connected), NULL);
-	guint idd_disc = g_signal_connect (device, "disconnected", G_CALLBACK (gps_location_disconnected), NULL);
-	
-	guint idc_run = g_signal_connect (control, "gpsd_running", G_CALLBACK (gps_location_started), NULL);
-	guint idc_stop = g_signal_connect (control, "gpsd_stopped", G_CALLBACK (gps_location_stopped), NULL);
-	guint idc_error = g_signal_connect (control, "error", G_CALLBACK (gps_location_error), NULL);
-	
-	printf("[Info] Entering the main loop event\n");
-        context=g_main_context_new();
-  	loop = g_main_loop_new(context, FALSE);
-	/* g_timeout_add(1000, (GSourceFunc)callback, NULL); */
- 	g_main_loop_run(loop);
- 	
- 	/* Exiting */
- 	g_main_loop_unref(loop);
- 	g_main_context_unref(context);
-	
-	/* Disconnect signal */
-	if(control->can_control)
-		location_gpsd_control_stop(control);
-		
-	g_signal_handler_disconnect(device, idd_changed);
-	g_signal_handler_disconnect(device, idd_con);
-	g_signal_handler_disconnect(device, idd_disc);
-	
-	g_signal_handler_disconnect(control, idc_run);
-	g_signal_handler_disconnect(control, idc_stop);
-	g_signal_handler_disconnect(control, idc_error);
-
-	/*	Device control cleanup */
-	g_object_unref(control);	
-	g_object_unref(device);
-
-	return 0;
-}
-#endif
