@@ -351,75 +351,6 @@ void TimeLine::Draw(int x, int y, size_t w, size_t h)
   lastDrawnBottom=bottom;
 }
 
-bool TimeLine::HandleMouseEvent(const Lum::OS::MouseEvent& event)
-{
-  /*
-  if (event.type==Lum::OS::MouseEvent::down &&
-     PointIsIn(event) &&
-     event.button==Lum::OS::MouseEvent::button1) {
-    gestureStartX=event.x;
-    gestureStartY=event.y;
-
-    if (GetWindow()->IsDoubleClicked()) {
-      Lum::Base::Calendar day=data.GetStartDate();
-
-      day.AddDays((event.x-x+hAdjustment->GetTop()-1)/GetDayWidth());
-
-      date->Set(day);
-      currentTab->Set(1);
-    }
-    return true;
-  }
-  else if (event.IsGrabEnd() && event.button==Lum::OS::MouseEvent::button1) {
-
-    if (std::abs(gestureStartX-event.x)>=std::abs(gestureStartY-event.y) ||
-        std::abs(gestureStartY-event.y)<(height*10)/100) {
-      return false;
-    }
-
-    if (event.y>gestureStartY) {
-      // down
-      mode=(Mode)((mode+1)%(modeMax+1));
-    }
-    else {
-      // up
-      if (mode==0) {
-        mode=modeMax;
-      }
-      else {
-        mode=(Mode)((mode-1)%(modeMax+1));
-      }
-    }
-
-    Draw(oX,oY,oWidth,oHeight);
-  }
-  else if (event.type==Lum::OS::MouseEvent::down &&
-     PointIsIn(event) &&
-     event.button==Lum::OS::MouseEvent::button4) {
-   if (mode==0) {
-     mode=modeMax;
-   }
-   else {
-     mode=(Mode)((mode-1)%(modeMax+1));
-   }
-
-   Draw(oX,oY,oWidth,oHeight);
-
-   return true;
-  }
-  else if (event.type==Lum::OS::MouseEvent::down &&
-          PointIsIn(event) &&
-          event.button==Lum::OS::MouseEvent::button5) {
-   mode=(Mode)((mode+1)%(modeMax+1));
-
-   Draw(oX,oY,oWidth,oHeight);
-
-   return true;
-  }*/
-
-  return false;
-}
-
 void TimeLine::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
 {
   if (model==hAdjustment->GetTopModel()) {
